/*

 buffer.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "buffer.h"
#include <string.h>
#include <stdlib.h>
#include "partition.h"
#include "arguments.h"
#include "path.h"

#ifdef WIN32
#define SYSTEMNAME "WIN32"
#elif defined __DJGPP__
#define SYSTEMNAME "DOS32"
#else
#define SYSTEMNAME "LINUX"
#endif

void buffer_cookeddir(Buffer *buffer, Directory *directory, int partition, const Petscii *outname) {
    Directory_entry dirent;
    unsigned char dirline[32];
    Petscii name[17], type[4] = {'*', 0};
    unsigned int sum = 0;
    int l;

    if (outname != NULL) convertfilename(outname, '=', name, type, NULL);
    if (!name[0]) strcpy((char *)name, "*");

    memcpy(&dirline[0x00], "\1\4\1\1\1\0\22\42", 8);
    dirline[0x04] = partition != 0 ? partition : partition_get_current();
    memset(&dirline[0x08], 32, 16);
    memcpy(&dirline[0x18], "\42\40" SYSTEMNAME "\0", 8);
    if (!buffer->data) {
        buffer->data = (unsigned char *)malloc(65536);
        if (!buffer->data) exit(1); //out of memory?!
    }
    for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];

    while (directory_read(directory, &dirent))
    {
        if (!matchname(dirent.name, name)) continue;
        if (!matchname(dirent.filetype, type)) continue;

        sum += directory_entry_cook(&dirent, dirline);
        if (sum > 65535) sum = 65535;
        for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
        if (buffer->length >= 65536 - 64) break;
    }

    dirline[0x00] = dirline[0x01] = 1;
    dirline[0x02] = sum;
    dirline[0x03] = sum >> 8;
    memcpy(&dirline[0x04], "BLOCKS USED.             \0\0\0", 28);

    for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
}

void buffer_rawdir(Buffer *buffer, Directory *directory) {
    unsigned char dirline[32];
    int l;

    if (!buffer->data) {
        buffer->data = (unsigned char *)malloc(65536);
        if (!buffer->data) exit(1); //out of memory?!
    }
    buffer->data[buffer->length++] = 'I';

    memset(&dirline[0], 32, 16);
    memset(&dirline[16], 0, 8);
    dirline[24] = A_DIR;
    memcpy(&dirline[25], "DIR", 3);
    memset(&dirline[28], 0, 4);
    for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];

    while (directory_rawread(directory, dirline))
    {
        for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
        if (buffer->length >= 65536 - 32) break;
    }
}

void buffer_partition(Buffer *buffer) {
    unsigned char dirline[32];
    int l, i, n = 0;
    memcpy(&dirline[0x00], "\1\4\1\1\377\0\22\42", 8);
    memcpy(&dirline[0x08], "IDESERVD " VERSION2 "  ", 16);
    memcpy(&dirline[0x18], "\42\40" SYSTEMNAME "\0", 8);
    if (!buffer->data) {
        buffer->data = (unsigned char *)malloc(65536);
        if (!buffer->data) exit(1); //out of memory?!
    }
    for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
    for (i = 1; i < 255; i++) {
        const Petscii *name = partition_get_name(i);
        if (!name) continue;
        dirline[0x00] = dirline[0x01] = 1;
        dirline[0x02] = i;
        dirline[0x03] = 0;
        memcpy(&dirline[0x04], "   \42\42                 NET  \0", 28);
        memcpy(&dirline[0x08], name, strlen((char *)name));
        dirline[0x08 + strlen((char *)name)] = '"';
        for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
        n++;
        if (buffer->length >= 65536 - 64) break;
    }
    dirline[0x00] = dirline[0x01] = 1;
    dirline[0x02] = n;
    dirline[0x03] = 0;
    memcpy(&dirline[0x04], "PARTITIONS.              \0\0\0", 28);
    for (l = 0; l < 32; l++) buffer->data[buffer->length++] = dirline[l];
}

